/*
** Module   : MenuItemWidget.java
** Abstract : server side menu item widget implementation
**
** Copyright (c) 2014-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 VIG 20141118          Created initial version.
*/
package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * This class represents menu-item widget.
 */
@LegacyResource(resource = LegacyResource.MENU_ITEM)
public class MenuItemWidget
extends GenericWidget<MenuItemConfig>
{
   /**
    * Hot key for menu calling. Set by adding '&' to label.
    * The key is the first character after '&' or first 
    * character of the whole label if there is no '&' in it.
    */
   private Character mnemonic;
   
   /**
    * READ-ONLY attr. Specifies that this menu item is read-only text. 
    * The user cannot choose this item.
    */
   private boolean readOnly = false;
   
   /**
    * TOGGLE-BOX attr. Specifies that the menu item is displayed as a checkbox 
    * that the user can toggle on or off. In environments that do not support 
    * this option, it is ignored.
    */
   private boolean toggleBox = false;
   
   /**
    * ACCELERATOR attr. Specifies a keyboard accelerator for this menu item. 
    * The value keylabel must be character-string expression that
    * evaluates to a valid key label recognized by Progress, such as a, F1, or ALT-SHIFT-F1.
    */
   private String accelerator;
   
   /**
    * Default constructor for dynamic menu-items  
    */
   public MenuItemWidget()
   {
      super(true, new MenuItemConfig());
   }

   /**
    * Default constructor for static menu-item
    * 
    * @param    dynamic
    *           Flag indicating if this is a static or dynamic resource.
    */
   public MenuItemWidget(String name, String label)
   {
      super(false, new MenuItemConfig());
      name(name);
      setLabel(label);
   }

   /**
    * Sets the label text, parsing mnemonic.
    * 
    * TODO: find out what is going on with undocumented feature of very long label
    * See parent's javadoc.
    * 
    * @param    label
    *           The new label text.
    */
   @Override
   public void setLabel(String label)
   {
      config.setDynamicLabel(label);
      
      int idx = label.indexOf('&');
      
      // TODO find out what mnemonic will be if '&' is the last symbol.
      if (idx < 0 || idx == label.length() - 1) 
      {
         mnemonic = label.charAt(0);
      }
      else
      {
         mnemonic = label.charAt(idx + 1);
      }
      
      // TODO It seems that pushScreenDefinition is not needed here 
      //      because the menu-item is not in a frame. Not sure because
      //      a popup menu can be owned by widget, for example, by button.
   }

   /**
    * Get MENU-ITEM mnemonic key.
    * 
    * @return  Mnemonic character.
    *          At some moment of dynamic menu-item construction it can be null. 
    */
   public Character getMnemonic()
   {
      return mnemonic;
   }
   
   /**
    * Set MENU-ITEM parent: MENU or SUB-MENU
    * 
    * @param   p
    *          Menu or Sub-Menu.
    */
   @Override
   public void setParent(GenericWidget<?> p)
   {
      // TODO implement if needed
   }

   /**
    * Checks whether the widget is write-protected.
    * 
    * @return   <code>true</code> if the widget is write-protected.
    */
   @Override
   public logical isReadOnly()
   {
      return new logical(readOnly);
   }

   /**
    * Sets the READ-ONLY attribute of the widget.
    * 
    * @param    r
    *           <code>true</code> if the widget should be write-protected.
    */
   @Override
   public void setReadOnly(logical r)
   {
      if (r != null)
         setReadOnly(r.booleanValue());
   }
   
   /**
    * Sets the READ-ONLY attribute of the widget.
    * 
    * @param    r
    *           <code>true</code> if the widget should be write-protected.
    */
   public void setReadOnly(boolean r)
   {
      readOnly = r;
   }

   /**
    * Sets new value of SENSITIVE attribute.
    *
    * @param disabled
    *        New value of SENSITIVE attribute. <code>TRUE</code> means <code>SENSITIVE=FALSE</code>
    *        and vice versa:  <code>FALSE</code> means <code>SENSITIVE=TRUE</code>.
    */
   public void setDisabled(boolean disabled) 
   {
      this.setSensitive(!disabled);
   }

   /**
    * Gets the TOGGLE-BOX attribute value.
    * 
    * @return  See above.
    */
   public boolean isToggleBox()
   {
      return toggleBox;
   }

   /**
    * Sets the TOGGLE-BOX attribute of the menu-item.
    * 
    * @param    toggleBox
    *           <code>true</code> if the menu-item should have a checkbox.
    */
   public void setToggleBox(boolean toggleBox)
   {
      this.toggleBox = toggleBox;
   }
   
   /**
    * Gets the display color writable attribute.
    *
    * @return   The current value of the display color attribute.
    */
   @Override
   public integer getDColor()
   {
      int color = config.dcolor.color();
      return (color == -1) ? new integer() : new integer(color);
   }
   
   /**
    * Sets the display color.
    *
    * @param    param
    *           The display color.
    */
   public void setDColor(NumberType param)
   {
      config.dcolor = new ColorSpec(param.intValue()).convert();
   }

   /**
    * Sets the display color.
    * 
    * @param    displayColor
    *           The display color.
    */
   public void setDColor(Color displayColor)
   {
      config.dcolor = displayColor;
   }

   /**
    * Sets the display color.
    * 
    * @param    displayColor
    *           The display color.
    */
   public void setDColor(int displayColor)
   {
      config.dcolor = new ColorSpec(displayColor).convert();
   }

   /**
    * Get foreground color.
    * 
    * @return   See above.
    */
   @Override
   public integer getFgColor()
   {
      return (config.fgcolor == -1) ? new integer() : new integer(config.fgcolor);
   }

   /**
    * Set foreground color.
    * 
    * @param   fgcolor
    *          Color number.
    */
   @Override
   public void setFgColor(integer fgcolor)
   {
      config.fgcolor = fgcolor.intValue();
   }

   /**
    * Set foreground color.
    * 
    * @param   fgcolor
    *          Color number.
    */
   public void setFgColor(int fgcolor)
   {
      config.fgcolor = fgcolor;
   }

   /**
    * Get background color.
    * 
    * @return   See above.
    */
   @Override
   public integer getBgColor()
   {
      return (config.bgcolor == -1) ? new integer() : new integer(config.bgcolor);
   }

   /**
    * Set background color.
    * 
    * @param   bgcolor
    *          Color number.
    */
   @Override
   public void setBgColor(integer bgcolor)
   {
      config.bgcolor = bgcolor.intValue();
   }

   /**
    * Set background color.
    * 
    * @param   bgcolor
    *          Color number.
    */
   public void setBgColor(int bgcolor)
   {
      config.bgcolor = bgcolor;
   }

   /**
    * Gets the input (prompt/edit mode) color.
    *
    * @return   The current value of the display color attribute.
    */
   @Override
   public integer getPfColor()
   {
      int color = config.pfcolor.color();
      return (color == -1) ? new integer() : new integer(color);
   }
   
   /**
    * Sets the input (prompt/edit mode) color.
    *
    * @param    param
    *           The display color.
    */
   public void setPfColor(NumberType param)
   {
      config.pfcolor = new ColorSpec(param.intValue()).convert();
   }

   /**
    * Sets the input (prompt/edit mode) color.
    * 
    * @param    displayColor
    *           The display color.
    */
   public void setPfColor(Color displayColor)
   {
      config.pfcolor = displayColor;
   }

   /**
    * Sets the input (prompt/edit mode) color.
    * 
    * @param    displayColor
    *           The display color.
    */
   public void setPfColor(int displayColor)
   {
      config.pfcolor = new ColorSpec(displayColor).convert();
   }

   /**
    * Gets this menu-item accelerator.
    * 
    * @return  See above.
    */
   public String getAccelerator()
   {
      return accelerator;
   }

   /**
    * Sets this menu-item accelerator.
    * 
    * @param  accelerator.
    *         The valid keylabel, that will be target accelerator.
    */
   public void setAccelerator(String accelerator)
   {
      this.accelerator = accelerator;
   }

}